// set the dimensions and margins of the graph
const margin = {
    top: 50,
    right: 400,
    bottom: 45,
    left: 75,
  },
  width = 945 - margin.left - margin.right,
  height = 472 - margin.top - margin.bottom;

// append the svg object to the body of the page
const svg = d3
  .select('#bubble_chart')
  .append('svg')
  .attr('width', width + margin.left + margin.right)
  .attr('height', height + margin.top + margin.bottom)
  .append('g')
  .attr(
    'transform',
    `translate(${margin.left},${margin.top})`,
  );

// read the data
d3.csv('iris.csv').then((data) => {
  // add chart title
  svg
    .append('text')
    .attr('x', width / 2)
    .attr('y', -20)
    .attr('text-anchor', 'middle')
    .style('font-size', '22px')
    .text(
      'Correlation Between Sepal Length, Sepal Width, and Petal Length',
    );

  // add x axis
  const x = d3
    .scaleLinear()
    .domain([4, 8])
    .range([0, width]);
  svg
    .append('g')
    .attr('transform', `translate(0, ${height})`)
    .call(d3.axisBottom(x));

  // add x axis label
  svg
    .append('text')
    .attr('text-anchor', 'middle')
    .attr('x', width / 2)
    .attr('y', height + 40)
    .text('Sepal Length');

  // add y axis
  const y = d3
    .scaleLinear()
    .domain([2, 4.5])
    .range([height, 0]);
  svg.append('g').call(d3.axisLeft(y));

  // add y axis label:
  svg
    .append('text')
    .attr('text-anchor', 'middle')
    .attr('x', -height / 2)
    .attr('y', -50)
    .text('Sepal Width')
    .attr('transform', 'rotate(-90)');

  // add a scale for bubble size
  const z = d3.scaleLinear().domain([1, 7]).range([4, 40]);

  // add a scale for bubble color
  const species = Array.from(
    new Set(data.map((d) => d.species)),
  );
  const speciesColors = species.map((el, i, arr) =>
    d3.interpolateTurbo(i / arr.length),
  );
  const myColor = d3
    .scaleOrdinal()
    .domain(species)
    .range(speciesColors);

  // -1- create a tooltip div that is hidden by default:
  const tooltip = d3
    .select('#bubble_chart')
    .append('div')
    .style('opacity', 0)
    .attr('class', 'tooltip')
    .style('background-color', 'black')
    .style('border-radius', '5px')
    .style('padding', '10px')
    .style('color', 'white')
    .style('font-family', 'sans-serif');

  // -2- create 3 functions to show / update / hide the tooltip
  const showTooltip = (event, d) => {
    tooltip.transition().duration(200);
    tooltip
      .style('opacity', 1)
      .html(
        'Species: ' +
          d.species +
          '<br/>Petal Length: ' +
          d.petal_length,
      )
      .style('left', event.x + 25 + 'px')
      .style('top', event.y + 25 + 'px');
  };
  const moveTooltip = (event, d) => {
    tooltip
      .style('left', event.x + 25 + 'px')
      .style('top', event.y + 25 + 'px');
  };
  const hideTooltip = () => {
    tooltip.transition().duration(200).style('opacity', 0);
  };

  // add dots
  svg
    .append('g')
    .selectAll('dot')
    .data(data)
    .join('circle')
    .attr('class', 'bubbles')
    .attr('cx', (d) => x(d.sepal_length))
    .attr('cy', (d) => y(d.sepal_width))
    .attr('r', (d) => z(d.petal_length))
    .style('fill', (d) => myColor(d.species))
    .attr('stroke', 'black')
    .on('mouseover', showTooltip)
    .on('mousemove', moveTooltip)
    .on('mouseleave', hideTooltip);

  // Legend for species colors
  const legendSpace = 20; // spacing for legend
  species.forEach((s, i) => {
    svg
      .append('circle')
      .attr('cx', width + margin.right - 120)
      .attr('cy', i * legendSpace)
      .attr('r', 7)
      .style('fill', myColor(s));

    svg
      .append('text')
      .attr('x', width + margin.right - 100)
      .attr('y', i * legendSpace + 5)
      .text(s)
      .style('font-size', '15px')
      .attr('alignment-baseline', 'middle');
  });

  // Optionally add more legends for petal length size, etc.
});
